/***************************************************
 This is a very hacky Kittly-Litterbox-Wifi-Scale implementation
 (C) Daniel Kirstenpfad 2016

 It initially started as my first Arduino / ESP8266 project and this is why it looks like it looks.
 To make it work you obviously have to change the belows Wifi password and ssid to yours. Also it
 will help to set the correct MQTT Server.

 It's loosely based upon:

  Adafruit MQTT Library ESP8266 Example

  Must use ESP8266 Arduino from:
    https://github.com/esp8266/Arduino

  Works great with Adafruit's Huzzah ESP board & Feather
  ----> https://www.adafruit.com/product/2471
  ----> https://www.adafruit.com/products/2821

  Adafruit invests time and resources providing this open source code,
  please support Adafruit and open-source hardware by purchasing
  products from Adafruit!

  Written by Tony DiCola for Adafruit Industries.
  MIT license, all text above must be included in any redistribution
 ****************************************************/
#include <ESP8266WiFi.h>
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_Client.h"
#include "HX711.h"

/************************* WiFi Access Point *********************************/

#define WLAN_SSID       "wifi_ssid"
#define WLAN_PASS       "wifi_password"

/************************* Adafruit.io Setup *********************************/

#define AIO_SERVER      "192.168.178.30"
#define AIO_SERVERPORT  1883                   // use 8883 for SSL
#define AIO_USERNAME    "user"
#define AIO_KEY         "key"

HX711 scale;

/************ Global State (you don't need to change this!) ******************/

// Create an ESP8266 WiFiClient class to connect to the MQTT server.
WiFiClient client;
// or... use WiFiFlientSecure for SSL
//WiFiClientSecure client;

// Store the MQTT server, username, and password in flash memory.
// This is required for using the Adafruit MQTT library.
const char MQTT_SERVER[] PROGMEM    = AIO_SERVER;
const char MQTT_USERNAME[] PROGMEM  = AIO_USERNAME;
const char MQTT_PASSWORD[] PROGMEM  = AIO_KEY;

// Setup the MQTT client class by passing in the WiFi client and MQTT server and login details.
Adafruit_MQTT_Client mqtt(&client, MQTT_SERVER, AIO_SERVERPORT, MQTT_USERNAME, MQTT_PASSWORD);

/****************************** Feeds ***************************************/

// Setup a feed called 'raw_weight_mqtt' for publishing.
// Notice MQTT paths for AIO follow the form: <username>/feeds/<feedname>
const char RAWWEIGHT_FEED[] PROGMEM = "house/stappenbach/bathroom_upstairs/litterbox/weight";
Adafruit_MQTT_Publish raw_weight_mqtt = Adafruit_MQTT_Publish(&mqtt, RAWWEIGHT_FEED);

const char CATWEIGHT_FEED[] PROGMEM = "house/stappenbach/bathroom_upstairs/litterbox/cat_weight";
Adafruit_MQTT_Publish cat_weight_mqtt = Adafruit_MQTT_Publish(&mqtt, CATWEIGHT_FEED);


// Setup a feed called 'tare' for subscribing to changes.
const char ONOFF_FEED[] PROGMEM = "house/stappenbach/bathroom_upstairs/litterbox/tare";
Adafruit_MQTT_Subscribe tare_scale_mqtt = Adafruit_MQTT_Subscribe(&mqtt, ONOFF_FEED);

/*************************** Sketch Code ************************************/

// Bug workaround for Arduino 1.6.6, it seems to need a function declaration
// for some reason (only affects ESP8266, likely an arduino-builder bug).
void MQTT_connect();

void setup() {
  pinMode(0, OUTPUT);
  Serial.begin(115200);
  delay(10);
  digitalWrite(0, LOW);

  // Connect to WiFi access point.
  Serial.println(); Serial.println();
  Serial.print("Connecting to ");
  Serial.println(WLAN_SSID);

  WiFi.begin(WLAN_SSID, WLAN_PASS);

   while (WiFi.status() != WL_CONNECTED) {
    digitalWrite(0, LOW);
    delay(500);
    digitalWrite(0, HIGH);
    delay(500);
    Serial.print(".");
  }

  Serial.println();
  Serial.println("WiFi connected");
  Serial.println("IP address: "); Serial.println(WiFi.localIP());

  pinMode(0, OUTPUT);
  digitalWrite(0, HIGH);

  // Ensure the connection to the MQTT server is alive (this will make the first
  // connection and automatically reconnect when disconnected).  See the MQTT_connect
  // function definition further below.
  MQTT_connect();

  // Setup MQTT subscription for tare feed.
  mqtt.subscribe(&tare_scale_mqtt);

  // -------------- HX711
  scale.begin(12, 14);

  Serial.print("calibrate");
  if (! raw_weight_mqtt.publish(0.f, 5)) {
    Serial.println(F("Failed"));
  } else {
    Serial.println(F("OK!"));
  }
  scale.set_scale();
  scale.tare();
  Serial.print("go");
  delay(1000);

  if (! raw_weight_mqtt.publish(scale.get_units(10), 5)) {
    Serial.println(F("Failed"));
  } else {
    Serial.println(F("OK!"));
  }

  scale.set_scale(-18721.f);
  scale.tare();               // reset the scale to 0

  //Serial.println("Readings:");
  // ---------------HX711

}

uint32_t x=0;
String tare;
float weight = 0.0;
float drift_weight = 0.0;
float diff_weight_drift = 0.0;
float cat_weight = 0.0;
bool cat_on_scale = false;

float diff_weight_threshold = 3.0;



void loop() {
  // Ensure the connection to the MQTT server is alive (this will make the first
  // connection and automatically reconnect when disconnected).  See the MQTT_connect
  // function definition further below.
  MQTT_connect();

  // this is our 'wait for incoming subscription packets' busy subloop
  // try to spend your time here

  Adafruit_MQTT_Subscribe *subscription;
  while ((subscription = mqtt.readSubscription(3000))) {
    if (subscription == &tare_scale_mqtt) {
      Serial.print(F("Got: "));
      scale.power_up();
      tare = String((char *)tare_scale_mqtt.lastread);
      scale.set_scale(tare.toFloat());
      scale.tare();
    }
  }

  scale.power_up();

if (!cat_on_scale)
{
  Serial.println("NOCOS");

  // do weight and diff when no cat is on scale...
  drift_weight = weight;
  weight = scale.get_units(20);
  diff_weight_drift = weight - drift_weight;

  if (diff_weight_drift >= diff_weight_threshold)
  {
    cat_on_scale = true;
    Serial.println("COS-ENTER");
  }

}
else
{
  // a cat is on the scale...
  Serial.println("COS");

  // increase the number of weights to average out
  weight = scale.get_units(100);

    if (weight >= diff_weight_threshold)
    {
      cat_weight = weight - drift_weight;
    }
    else
    {
      // reset drift_weight for later use...
      drift_weight = 0.0;

      // cat has left the scale
      Serial.println("COS-LEAVE");
      cat_on_scale = false;
      Serial.print("cat weight: \t");
      Serial.println(cat_weight, 5);

      // publish...
      if (! cat_weight_mqtt.publish(cat_weight, 5)) {
        Serial.println(F("publish failed!"));
      } else {
      //Serial.println(F("published!"));
      }
    }
}

  // correct for negative drift?
  if (drift_weight < 0) {
    drift_weight = 0.0;
    Serial.println("correcting drift_weight");
    //Serial.print("drift_weight: \t");
    //Serial.println(drift_weight, 5);
  }

  // correct for drifting
  weight = weight - drift_weight;
  Serial.print("weight: \t");
  Serial.println(weight, 5);
  // Now we can publish stuff!

  if (! raw_weight_mqtt.publish(weight, 5)) {
    Serial.println(F("publish failed!"));
  } else {
    //Serial.println(F("published!"));
  }

  scale.power_down();             // put the ADC in sleep mode
  //delay(200);
   // ping the server to keep the mqtt connection alive
  // NOT required if you are publishing once every KEEPALIVE seconds

  //if(! mqtt.ping()) {
  //  mqtt.disconnect();
  //}
}

// Function to connect and reconnect as necessary to the MQTT server.
// Should be called in the loop function and it will take care if connecting.
void MQTT_connect() {
  int8_t ret;

  // Stop if already connected.
  if (mqtt.connected()) {
    return;
  }

  Serial.print("Connecting to MQTT... ");

  uint8_t retries = 3;
  while ((ret = mqtt.connect()) != 0) { // connect will return 0 for connected
       Serial.println(mqtt.connectErrorString(ret));
       Serial.println("Retrying MQTT connection in 5 seconds...");
       mqtt.disconnect();
       delay(5000);  // wait 5 seconds
       retries--;
       if (retries == 0) {
         // basically die and wait for WDT to reset me
         while (1);
       }
  }
  Serial.println("MQTT Connected!");
}
